@@ -25,12 +25,13 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "u_private.h"
 #include "ulfius.h"
 
 /**
  * Splits the url to an array of char *
  */
-char ** ulfius_split_url(const char * prefix, const char * url) {
+static char ** ulfius_split_url(const char * prefix, const char * url) {
   char * saveptr = NULL, * cur_word = NULL, ** to_return = o_malloc(sizeof(char*)), * url_cpy = NULL, * url_cpy_addr = NULL;
   int counter = 1;
   
@@ -90,7 +91,7 @@ char ** ulfius_split_url(const char * prefix, const char * url) {
  * Compare two endoints by their priorities
  * Used by qsort to compare endpoints
  */
-int compare_endpoint_priorities(const void * a, const void * b) {
+static int compare_endpoint_priorities(const void * a, const void * b) {
   struct _u_endpoint * e1 = *(struct _u_endpoint **)a, * e2 = *(struct _u_endpoint **)b;
   
   if (e1->priority < e2->priority) {
@@ -102,6 +103,25 @@ int compare_endpoint_priorities(const void * a, const void * b) {
   }
 }
 
+/**
+ * ulfius_url_format_match
+ * return true if splitted_url matches splitted_url_format
+ * false otherwise
+ */
+static int ulfius_url_format_match(const char ** splitted_url, const char ** splitted_url_format) {
+  int i;
+  
+  for (i=0; splitted_url_format[i] != NULL; i++) {
+    if (splitted_url_format[i][0] == '*' && splitted_url_format[i+1] == NULL) {
+      return 1;
+    }
+    if (splitted_url[i] == NULL || (splitted_url_format[i][0] != '@' && splitted_url_format[i][0] != ':' && 0 != o_strcmp(splitted_url_format[i], splitted_url[i]))) {
+      return 0;
+    }
+  }
+  return (splitted_url[i] == NULL && splitted_url_format[i] == NULL);
+}
+
 /**
  * ulfius_endpoint_match
  * return the endpoint array matching the url called with the proper http method
@@ -148,25 +168,6 @@ struct _u_endpoint ** ulfius_endpoint_match(const char * method, const char * ur
   return endpoint_returned;
 }
 
-/**
- * ulfius_url_format_match
- * return true if splitted_url matches splitted_url_format
- * false otherwise
- */
-int ulfius_url_format_match(const char ** splitted_url, const char ** splitted_url_format) {
-  int i;
-  
-  for (i=0; splitted_url_format[i] != NULL; i++) {
-    if (splitted_url_format[i][0] == '*' && splitted_url_format[i+1] == NULL) {
-      return 1;
-    }
-    if (splitted_url[i] == NULL || (splitted_url_format[i][0] != '@' && splitted_url_format[i][0] != ':' && 0 != o_strcmp(splitted_url_format[i], splitted_url[i]))) {
-      return 0;
-    }
-  }
-  return (splitted_url[i] == NULL && splitted_url_format[i] == NULL);
-}
-
 /**
  * ulfius_parse_url
  * fills map with the keys/values defined in the url that are described in the endpoint format url
@@ -401,19 +402,19 @@ struct _u_request * ulfius_duplicate_request(const struct _u_request * request)
 #ifndef U_DISABLE_JANSSON
 /**
  * ulfius_set_json_body_request
- * Add a json_t binary_body to a response
+ * Add a json_t j_body to a response
  * return U_OK on success
  */
-int ulfius_set_json_body_request(struct _u_request * request, json_t * body) {
-  if (request != NULL && body != NULL) {
+int ulfius_set_json_body_request(struct _u_request * request, json_t * j_body) {
+  if (request != NULL && j_body != NULL && (json_is_array(j_body) || json_is_object(j_body))) {
     // Free all the bodies available
     o_free(request->binary_body);
     request->binary_body = NULL;
     request->binary_body_length = 0;
 
-    request->binary_body = (void*) json_dumps(body, JSON_COMPACT);
+    request->binary_body = (void*) json_dumps(j_body, JSON_COMPACT);
     if (request->binary_body == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for dest->binary_body");
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for request->binary_body");
       return U_ERROR_MEMORY;
     }
     request->binary_body_length = strlen((char*)request->binary_body);
@@ -429,10 +430,24 @@ int ulfius_set_json_body_request(struct _u_request * request, json_t * body) {
  * Get JSON structure from the request body if the request is valid
  * request: struct _u_request used
  * json_error: structure to store json_error_t if specified
- */
+ */  
 json_t * ulfius_get_json_body_request(const struct _u_request * request, json_error_t * json_error) {
   if (request != NULL && request->map_header != NULL && NULL != o_strstr(u_map_get_case(request->map_header, ULFIUS_HTTP_HEADER_CONTENT), ULFIUS_HTTP_ENCODING_JSON)) {
     return json_loadb(request->binary_body, request->binary_body_length, JSON_DECODE_ANY, json_error);
+  } else if (json_error != NULL) {
+    json_error->line     = 1;
+    json_error->position = 1;
+    snprintf(json_error->source, (JSON_ERROR_SOURCE_LENGTH - 1), "ulfius_get_json_body_request");
+    if (NULL == request) {
+      json_error->column = 7;
+      snprintf(json_error->text, (JSON_ERROR_TEXT_LENGTH - 1), "Request not set.");
+    } else if (NULL == request->map_header) {
+      json_error->column = 26;
+      snprintf(json_error->text, (JSON_ERROR_TEXT_LENGTH - 1), "Request header not set.");
+    } else if (NULL == o_strstr(u_map_get_case(request->map_header, ULFIUS_HTTP_HEADER_CONTENT), ULFIUS_HTTP_ENCODING_JSON)) {
+      json_error->column = 57;
+      snprintf(json_error->text, (JSON_ERROR_TEXT_LENGTH - 1), "HEADER content not valid. Expected containging '%s' in header - received '%s'.", ULFIUS_HTTP_ENCODING_JSON, u_map_get_case(request->map_header, ULFIUS_HTTP_HEADER_CONTENT));
+    }
   }
   return NULL;
 }
