@@ -23,6 +23,7 @@
  * 
  */
 
+#include "u_private.h"
 #include "ulfius.h"
 
 #ifndef U_DISABLE_WEBSOCKET
@@ -31,7 +32,6 @@
 #include <fcntl.h>
 #include <pthread.h>
 #include <unistd.h>
-#include <sys/poll.h>
 #include <gnutls/gnutls.h>
 
 /**
@@ -111,7 +111,7 @@ void * ulfius_thread_websocket(void * data) {
   
   if (websocket != NULL && websocket->websocket_manager != NULL) {
     pthread_mutexattr_init ( &mutexattr );
-    pthread_mutexattr_settype( &mutexattr, PTHREAD_MUTEX_RECURSIVE_NP );
+    pthread_mutexattr_settype( &mutexattr, PTHREAD_MUTEX_RECURSIVE );
     if (pthread_mutex_init(&(websocket->websocket_manager->read_lock), &mutexattr) != 0 || pthread_mutex_init(&(websocket->websocket_manager->write_lock), &mutexattr) != 0) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Impossible to initialize Mutex Lock for websocket");
       websocket->websocket_manager->connected = 0;
@@ -343,6 +343,7 @@ int ulfius_clear_websocket(struct _websocket * websocket) {
     }
     ulfius_instance_remove_websocket_active(websocket->instance, websocket);
     ulfius_clear_websocket_manager(websocket->websocket_manager);
+		ulfius_clean_request_full(websocket->request);
     o_free(websocket->websocket_manager);
     websocket->websocket_manager = NULL;
     o_free(websocket);
@@ -734,7 +735,7 @@ int ulfius_instance_remove_websocket_active(struct _u_instance * instance, struc
         }
         ((struct _websocket_handler *)instance->websocket_handler)->nb_websocket_active--;
         pthread_mutex_lock(&((struct _websocket_handler *)instance->websocket_handler)->websocket_close_lock);
-        pthread_cond_signal(&((struct _websocket_handler *)instance->websocket_handler)->websocket_close_cond);
+        pthread_cond_broadcast(&((struct _websocket_handler *)instance->websocket_handler)->websocket_close_cond);
         pthread_mutex_unlock(&((struct _websocket_handler *)instance->websocket_handler)->websocket_close_lock);
         return U_OK;
       }
