@@ -27,7 +27,8 @@
 #define __ULFIUS_H__
 
 /** External dependencies **/
-#include <sys/poll.h>
+#include <poll.h>
+#include <pthread.h>
 #include <microhttpd.h>
 
 #if (MHD_VERSION < 0x00095300) && !defined(U_DISABLE_WEBSOCKET)
@@ -55,7 +56,7 @@
 #define U_ERROR_LIBCURL      5 // Error in libcurl execution
 #define U_ERROR_NOT_FOUND    6 // Something was not found
 
-#define ULFIUS_VERSION 2.1
+#define ULFIUS_VERSION 2.2.4
 
 #define U_CALLBACK_CONTINUE     0
 #define U_CALLBACK_COMPLETE     1
@@ -84,7 +85,7 @@ struct _u_cookie {
   char * key;
   char * value;
   char * expires;
-  uint   max_age;
+  unsigned int   max_age;
   char * domain;
   char * path;
   int    secure;
@@ -117,7 +118,7 @@ struct _u_request {
   char *               http_protocol;
   char *               http_verb;
   char *               http_url;
-	char *               proxy;
+  char *               proxy;
   int                  check_server_certificate;
   long                 timeout;
   struct sockaddr *    client_address;
@@ -193,7 +194,7 @@ struct _u_endpoint {
   char * http_method;
   char * url_prefix;
   char * url_format;
-  uint   priority;
+  unsigned int   priority;
   int (* callback_function)(const struct _u_request * request, // Input parameters (set by the framework)
                             struct _u_response * response,     // Output parameters (set by the user)
                             void * user_data);
@@ -222,7 +223,7 @@ struct _u_endpoint {
 struct _u_instance {
   struct MHD_Daemon          *  mhd_daemon;
   int                           status;
-  uint                          port;
+  unsigned int                          port;
   struct sockaddr_in          * bind_address;
   int                           nb_endpoints;
   char                        * default_auth_realm;
@@ -232,16 +233,27 @@ struct _u_instance {
   size_t                        max_post_param_size;
   size_t                        max_post_body_size;
   void                        * websocket_handler;
+  int                        (* file_upload_callback) (const struct _u_request * request, 
+                                                       const char * key, 
+                                                       const char * filename, 
+                                                       const char * content_type, 
+                                                       const char * transfer_encoding, 
+                                                       const char * data, 
+                                                       uint64_t off, 
+                                                       size_t size, 
+                                                       void * cls);
+  void                        * file_upload_cls;
 };
 
 /**
  * Structures used to facilitate data manipulations (internal)
  */
 struct connection_info_struct {
+  struct _u_instance       * u_instance;
   struct MHD_PostProcessor * post_processor;
   int                        has_post_processor;
   int                        callback_first_iteration;
-  struct _u_request *        request;
+  struct _u_request        * request;
   size_t                     max_post_param_size;
   struct _u_map              map_url_initial;
 };
@@ -250,13 +262,18 @@ struct connection_info_struct {
  * Instance functions declarations
  **********************************/
 
+/**
+ * free data allocated by ulfius functions
+ */
+void u_free(void * data);
+
 /**
  * ulfius_init_instance
  * 
  * Initialize a struct _u_instance * with default values
  * return U_OK on success
  */
-int ulfius_init_instance(struct _u_instance * u_instance, uint port, struct sockaddr_in * bind_address, const char * default_auth_realm);
+int ulfius_init_instance(struct _u_instance * u_instance, unsigned int port, struct sockaddr_in * bind_address, const char * default_auth_realm);
 
 /**
  * ulfius_clean_instance
@@ -294,6 +311,37 @@ int ulfius_start_secure_framework(struct _u_instance * u_instance, const char *
  */
 int ulfius_stop_framework(struct _u_instance * u_instance);
 
+/**
+ * ulfius_set_upload_file_callback_function
+ * 
+ * Set the callback function to handle file upload
+ * Used to facilitate large files upload management
+ * The callback function file_upload_callback will be called
+ * multiple times, with the uploaded file in striped in parts
+ * 
+ * Warning: If this function is used, all the uploaded files
+ * for the instance will be managed via this function, and they
+ * will no longer be available in the struct _u_request in the
+ * ulfius callback function afterwards.
+ * 
+ * Thanks to Thad Phetteplace for the help on this feature
+ * 
+ * u_instance:    pointer to a struct _u_instance that describe its port and bind address
+ * file_upload_callback: Pointer to a callback function that will handle all file uploads
+ * cls: a pointer that will be passed to file_upload_callback each tim it's called
+ */
+int ulfius_set_upload_file_callback_function(struct _u_instance * u_instance,
+                                             int (* file_upload_callback) (const struct _u_request * request, 
+                                                                           const char * key, 
+                                                                           const char * filename, 
+                                                                           const char * content_type, 
+                                                                           const char * transfer_encoding, 
+                                                                           const char * data, 
+                                                                           uint64_t off, 
+                                                                           size_t size, 
+                                                                           void * cls),
+                                             void * cls);
+
 /***********************************
  * Endpoints functions declarations
  ***********************************/
@@ -328,7 +376,7 @@ int ulfius_add_endpoint_by_val(struct _u_instance * u_instance,
                                const char * http_method,
                                const char * url_prefix,
                                const char * url_format,
-                               uint priority,
+                               unsigned int priority,
                                int (* callback_function)(const struct _u_request * request, // Input parameters (set by the framework)
                                                          struct _u_response * response,     // Output parameters (set by the user)
                                                          void * user_data),
@@ -481,7 +529,7 @@ int ulfius_send_smtp_email(const char * host,
  * add a cookie to the cookie map
  * return U_OK on success
  */
-int ulfius_add_cookie_to_response(struct _u_response * response, const char * key, const char * value, const char * expires, const uint max_age, 
+int ulfius_add_cookie_to_response(struct _u_response * response, const char * key, const char * value, const char * expires, const unsigned int max_age, 
                       const char * domain, const char * path, const int secure, const int http_only);
 
 /**
@@ -497,21 +545,21 @@ int ulfius_add_header_to_response(struct _u_response * response, const char * ke
  * body must end with a '\0' character
  * return U_OK on success
  */
-int ulfius_set_string_body_response(struct _u_response * response, const uint status, const char * body);
+int ulfius_set_string_body_response(struct _u_response * response, const unsigned int status, const char * body);
 
 /**
  * ulfius_set_binary_body_response
  * Add a binary body to a response
  * return U_OK on success
  */
-int ulfius_set_binary_body_response(struct _u_response * response, const uint status, const char * body, const size_t length);
+int ulfius_set_binary_body_response(struct _u_response * response, const unsigned int status, const char * body, const size_t length);
 
 /**
  * ulfius_set_empty_body_response
  * Set an empty response with only a status
  * return U_OK on success
  */
-int ulfius_set_empty_body_response(struct _u_response * response, const uint status);
+int ulfius_set_empty_body_response(struct _u_response * response, const unsigned int status);
 
 /**
  * ulfius_set_stream_response
@@ -519,7 +567,7 @@ int ulfius_set_empty_body_response(struct _u_response * response, const uint sta
  * return U_OK on success
  */
 int ulfius_set_stream_response(struct _u_response * response, 
-                                const uint status,
+                                const unsigned int status,
                                 ssize_t (* stream_callback) (void * stream_user_data, uint64_t offset, char * out_buf, size_t max),
                                 void (* stream_callback_free) (void * stream_user_data),
                                 uint64_t stream_size,
@@ -613,17 +661,17 @@ json_t * ulfius_get_json_body_request(const struct _u_request * request, json_er
 
 /**
  * ulfius_set_json_body_request
- * Add a json_t body to a request
+ * Add a json_t j_body to a request
  * return U_OK on success
  */
-int ulfius_set_json_body_request(struct _u_request * request, json_t * body);
+int ulfius_set_json_body_request(struct _u_request * request, json_t * j_body);
 
 /**
  * ulfius_set_json_body_response
- * Add a json_t body to a response
+ * Add a json_t j_body to a response
  * return U_OK on success
  */
-int ulfius_set_json_body_response(struct _u_response * response, const uint status, const json_t * body);
+int ulfius_set_json_body_response(struct _u_response * response, const unsigned int status, const json_t * j_body);
 
 /**
  * ulfius_get_json_body_response
@@ -957,112 +1005,6 @@ void ulfius_clear_websocket_message(struct _websocket_message * message);
 
 #endif
 
-/**
- * free data allocated by ulfius functions
- */
-void u_free(void * data);
-
-/**********************************
- * Internal functions declarations
- **********************************/
-
-/**
- * ulfius_validate_instance
- * return true if u_instance has valid parameters, false otherwise
- */
-int ulfius_validate_instance(const struct _u_instance * u_instance);
-/**
- * ulfius_is_valid_endpoint
- * return true if the endpoind has valid parameters
- */
-int ulfius_is_valid_endpoint(const struct _u_endpoint * endpoint, int to_delete);
-
-/**
- * ulfius_validate_endpoint_list
- * return true if endpoint_list has valid parameters, false otherwise
- */
-int ulfius_validate_endpoint_list(const struct _u_endpoint * endpoint_list, int nb_endpoints);
-
-/**
- * ulfius_webservice_dispatcher
- * function executed by libmicrohttpd every time an HTTP call is made
- * return MHD_NO on error
- */
-int ulfius_webservice_dispatcher (void *cls, struct MHD_Connection *connection,
-                                  const char *url, const char *method,
-                                  const char *version, const char *upload_data,
-                                  size_t *upload_data_size, void **con_cls);
-/**
- * mhd_iterate_post_data
- * function used to iterate post parameters
- * return MHD_NO on error
- */
-int mhd_iterate_post_data (void *coninfo_cls, enum MHD_ValueKind kind, const char *key,
-                      const char *filename, const char *content_type,
-                      const char *transfer_encoding, const char *data, uint64_t off,
-                      size_t size);
-
-/**
- * mhd_request_completed
- * function used to clean data allocated after a web call is complete
- */
-void mhd_request_completed (void *cls, struct MHD_Connection *connection,
-                        void **con_cls, enum MHD_RequestTerminationCode toe);
-
-/**
- * ulfius_split_url
- * return an array of char based on the url words
- * returned value must be u_free'd after use
- */
-char ** ulfius_split_url(const char * prefix, const char * url);
-
-/**
- * Sort an array of struct _u_endpoint * using bubble sort algorithm
- */
-void sort_endpoint_list (struct _u_endpoint ** endpoint_list, int length);
-
-/**
- * ulfius_endpoint_match
- * return the endpoint array matching the url called with the proper http method
- * the returned array always has its last value to NULL
- * return NULL on memory error
- */
-struct _u_endpoint ** ulfius_endpoint_match(const char * method, const char * url, struct _u_endpoint * endpoint_list);
-
-/**
- * ulfius_url_format_match
- * return true if splitted_url matches splitted_url_format
- * false otherwise
- */
-int ulfius_url_format_match(const char ** splitted_url, const char ** splitted_url_format);
-
-/**
- * ulfius_parse_url
- * fills map with the keys/values defined in the url that are described in the endpoint format url
- * return U_OK on success
- */
-int ulfius_parse_url(const char * url, const struct _u_endpoint * endpoint, struct _u_map * map);
-
-/**
- * ulfius_set_response_header
- * adds headers defined in the response_map_header to the response
- * return the number of added headers, -1 on error
- */
-int ulfius_set_response_header(struct MHD_Response * response, const struct _u_map * response_map_header);
-
-/**
- * ulfius_set_response_cookie
- * adds cookies defined in the response_map_cookie
- * return the number of added headers, -1 on error
- */
-int ulfius_set_response_cookie(struct MHD_Response * mhd_response, const struct _u_response * response);
-
-/**
- * Add a cookie in the cookie map as defined in the RFC 6265
- * Returned value must be u_free'd after use
- */
-char * ulfius_get_cookie_header(const struct _u_cookie * cookie);
-
 /** Macro values **/
 #define ULFIUS_URL_SEPARATOR       "/"
 #define ULFIUS_HTTP_ENCODING_JSON  "application/json"
@@ -1119,106 +1061,6 @@ struct _websocket_handler {
   int                           pthread_init;
 };
 
-/**
- * Websocket callback function for MHD
- * Starts the websocket manager if set,
- * then sets a listening message loop
- * Complete the callback when the websocket is closed
- * The websocket can be closed by the client, the manager, the program, or on network disconnect
- */
-void ulfius_start_websocket_cb (void *cls,
-            struct MHD_Connection *connection,
-            void *con_cls,
-            const char *extra_in,
-            size_t extra_in_size,
-            MHD_socket sock,
-            struct MHD_UpgradeResponseHandle *urh);
-
-/**
- * Workaround to make sure a message, as long as it can be is complete sent
- */
-void ulfius_websocket_send_all(MHD_socket sock, const uint8_t * data, size_t len);
-
-/**
- * Centralise socket reading in this function
- * so if options or check must be done, it's done here instead of each read call
- */
-size_t ulfius_websocket_recv_all(MHD_socket sock, uint8_t * data, size_t len);
-
-/**
- * Generates a handhshake answer from the key given in parameter
- */
-int ulfius_generate_handshake_answer(const char * key, char * out_digest);
-
-/**
- * Close the websocket
- */
-int ulfius_close_websocket(struct _websocket * websocket);
-
-/**
- * Return a match list between two list of items
- * If match is NULL, then return source duplicate
- * Returned value must be u_free'd after use
- */
-char * ulfius_check_list_match(const char * source, const char * match);
-
-/**
- * Initialize a websocket message list
- * Return U_OK on success
- */
-int ulfius_init_websocket_message_list(struct _websocket_message_list * message_list);
-
-/**
- * Clear data of a websocket message list
- */
-void ulfius_clear_websocket_message_list(struct _websocket_message_list * message_list);
-
-/**
- * Append a message in a message list
- * Return U_OK on success
- */
-int ulfius_push_websocket_message(struct _websocket_message_list * message_list, struct _websocket_message * message);
-
-/**
- * Clear data of a websocket
- */
-int ulfius_clear_websocket(struct _websocket * websocket);
-
-/**
- * Clear data of a websocket_manager
- */
-void ulfius_clear_websocket_manager(struct _websocket_manager * websocket_manager);
-
-/**
- * Read and parse a new message from the websocket
- * Return the opcode of the new websocket, U_WEBSOCKET_OPCODE_NONE if no message arrived, or U_WEBSOCKET_OPCODE_ERROR on error
- * Sets the new message in the message variable if available
- */
-int ulfius_read_incoming_message(struct _websocket_manager * websocket_manager, struct _websocket_message ** message);
-
-/**
- * Run the websocket manager in a separated detached thread
- */
-void * ulfius_thread_websocket_manager_run(void * args);
-
-/**
- * Send a message in the websocket without lock
- * Return U_OK on success
- */
-int ulfius_websocket_send_message_nolock(struct _websocket_manager * websocket_manager,
-                                  const uint8_t opcode,
-                                  const uint64_t data_len,
-                                  const char * data);
-
- /**
- * Add a websocket in the list of active websockets of the instance
- */
-int ulfius_instance_add_websocket_active(struct _u_instance * instance, struct _websocket * websocket);
-
-/**
- * Remove a websocket from the list of active websockets of the instance
- */
-int ulfius_instance_remove_websocket_active(struct _u_instance * instance, struct _websocket * websocket); 
 #endif // U_DISABLE_WEBSOCKET
 
 #endif // __ULFIUS_H__
